package benchmark.random;

import static util.parser.InstanceTokens.CONSTRAINT_MODEL;
import static util.parser.InstanceTokens.DisCSP;
import static util.parser.InstanceTokens.FORMAT;
import static util.parser.InstanceTokens.MODEL;
import static util.parser.InstanceTokens.NAME;
import static util.parser.InstanceTokens.NB_AGENTS;
import static util.parser.InstanceTokens.PRESENTATION;
import static util.parser.InstanceTokens.TKC;
import static util.parser.InstanceTokens.TYPE;
import static util.parser.InstanceTokens.XDisCSP_1_0;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Random;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import gui.GuiSimulator;
import util.Commands;
import util.Logger;

public class RBCDisCSPGenerator extends GuiSimulator {

	public static final String CONSTRAINTS_DENSITY = "constraintsDensity";
	public static final String TIGHTNESS = "tightness";
	public static final String P1 = "p1";
	public static final String P2 = "p2";
	public static final String MIN_P2 = "minP2";
	public static final String MAX_P2 = "maxP2";
	public static final String STEP_P2 = "stepP2";

	public static final String fixKeys = P2;
	public static int nbHeaderLine = 4;

	private int numAgents; // number of agents
	private int numVarsPerAgent; // number of variables per agent
	private int nbVariables;
	private int domainSize; // domain size

	private int Cx, minCx = 0, maxCx = 0, stepCx = 0; // number of connections
	private int CpCx, minCpCx = 0, maxCpCx = 0, stepCpCx = 0; // number of constraints
	// per connection
	private int p1, minP1 = 0, maxP1 = 0, stepP1 = 0; // number of local constraint per
	// agent
	private int p2, minP2 = 0, maxP2 = 0, stepP2 = 0; // Tightness (number of
	// infeasible/feasible tuples)

	private int nbConstraintsPerConnection, nbConnections, nbConstraints, tightness, nbInstances = 0;
	private Element instance = null;
	private Document document;
	private String path;
	private int nbIntraAgentConstraints;

	public RBCDisCSPGenerator(File directory, int nbInstances, int nbAgent, int nbVarsPerAgent, int domainSize) {
		if (!directory.exists()) {
			directory.mkdir();
		}
		this.path = directory.getPath();
		this.nbInstances = nbInstances;
		this.numAgents = nbAgent;
		this.numVarsPerAgent = nbVarsPerAgent;
		this.nbVariables = nbAgent * nbVarsPerAgent;
		this.domainSize = domainSize;

	}

	/**
	 * @param filePath
	 */
	public RBCDisCSPGenerator() {
		super();
	}

	public void setConnection(double minCx, double maxCx, double stepCx) {
		this.minCx = (int) (minCx * 100);
		this.maxCx = (int) (maxCx * 100);
		this.stepCx = (int) (stepCx * 100);
	}

	public void setConstraintsPerConnection(double minCpCx, double maxCpCx, double stepCpCx) {
		this.minCpCx = (int) (minCpCx * 100);
		this.maxCpCx = (int) (maxCpCx * 100);
		this.stepCpCx = (int) (stepCpCx * 100);
	}

	public void setLocalConstraintDensity(double minP1, double maxP1, double stepP1) {
		this.minP1 = (int) (minP1 * 100);
		this.maxP1 = (int) (maxP1 * 100);
		this.stepP1 = (int) (stepP1 * 100);
	}

	public void setTightness(double minP2, double maxP2, double stepP2) {
		this.minP2 = (int) (minP2 * 100);
		this.maxP2 = (int) (maxP2 * 100);
		this.stepP2 = (int) (stepP2 * 100);
	}

	/**
	 * @param file
	 */
	void save(String file) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(this.document, new FileOutputStream(file));
			Process p = Runtime.getRuntime().exec("bzip2 " + file);
			p.waitFor();
			p.exitValue();
			p.destroy();
		} catch (Exception e) {}
	}

	private Element getPresentation(int i) {
		Element presentation = new Element(PRESENTATION);
		presentation.setAttribute(NAME, "ComplexDisCSP_" + i);
		presentation.setAttribute(MODEL, Commands.COMPLEX);
		presentation.setAttribute(CONSTRAINT_MODEL, TKC);
		presentation.setAttribute(FORMAT, XDisCSP_1_0);
		presentation.setAttribute(TYPE, DisCSP);
		return presentation;
	}

	/**
	 * 
	 * @param instance
	 * @param presentation
	 * @param agents
	 * @param domains
	 * @param variables
	 * @param nbI
	 */
	public void write(Element presentation, Element agents, Element domains, Element variables, int nbInstance) {
		presentation = this.getPresentation(nbInstance + 1);
		agents = new Element("agents");
		domains = new Element("domains");
		variables = new Element("variables");

		this.instance.addContent(presentation);
		this.instance.addContent(agents);
		this.instance.addContent(domains);
		this.instance.addContent(variables);

		agents.setAttribute(NB_AGENTS, "" + this.numAgents);
		for (int ag = 1 ; ag <= this.numAgents ; ag++) {
			Element agent = new Element("agent");
			agent.setAttribute("name", "A" + ag);
			agent.setAttribute("id", "" + ag);
			agent.setAttribute("description", "");
			agents.addContent(agent);
		}

		domains.setAttribute("nbDomains", "1");
		Element domain = new Element("domain");
		domain.setAttribute("name", "D1");
		domain.setAttribute("nbValues", "" + domainSize);
		domain.setText("0.." + "" + (this.domainSize - 1));
		domains.addContent(domain);

		variables.setAttribute("nbVariables", "" + this.nbVariables);
		for (int ag = 1 ; ag <= this.numAgents ; ag++) {
			for (int i = 0 ; i < this.numVarsPerAgent ; i++) {
				Element variable = new Element("variable");
				variable.setAttribute("agent", "A" + ag);
				variable.setAttribute("name", "X" + ag + "." + i);
				variable.setAttribute("id", "" + i);
				variable.setAttribute("domain", "D1");
				variable.setAttribute("description", "A" + ag);
				variables.addContent(variable);
			}
		}
	}

	/**
	 * 
	 */
	public void generate() {
		Element configuration = new Element("conf");
		configuration.setAttribute("type", Commands.ComplexRandomTKC);
		configuration.setAttribute("format", "" + Commands.COMPLEX);
		configuration.setAttribute(NB_AGENTS, "" + this.numAgents);
		configuration.setAttribute("nbVarsPerAgents", "" + this.numVarsPerAgent);
		configuration.setAttribute("domainSize", "" + this.domainSize);
		String name = "ComplexDisCSP_";
		configuration.setAttribute("name", name);
		Document confdocument = new Document(configuration);
		for (p1 = this.minP1 ; p1 <= this.maxP1 ; p1 += this.stepP1) {// loop over
			// local
			// constraint
			// density
			double nIrac = (p1 / 100. * (double) this.numVarsPerAgent * (this.numVarsPerAgent - 1) / 2);
			this.nbIntraAgentConstraints = (int) Math.floor(nIrac);
			for (CpCx = this.minCpCx ; CpCx <= this.maxCpCx ; CpCx += this.stepCpCx) { // loop
				// over
				// number
				// of
				this.nbConstraintsPerConnection = (int) Math.floor(CpCx / 100. * this.numVarsPerAgent * this.numVarsPerAgent); // constraint
				// per
				// connection
				String subDir = "0." + ((p1 < 10)? "0" + p1 : p1) + "_0." + ((CpCx < 10)? "0" + CpCx : CpCx);
				File directory = new File(this.path + File.separator + subDir);
				if (!directory.exists()) {
					directory.mkdir();
				}
				for (Cx = this.minCx ; Cx <= this.maxCx ; Cx += this.stepCx) { // loop
					// over
					// connections
					this.nbConnections = (int) Math.floor((Cx / 100. * (double) this.numAgents * (this.numAgents - 1) / 2));
					for (p2 = this.minP2 ; p2 <= this.maxP2 ; p2 += this.stepP2) {// loop
						// over
						// Tightness
						this.tightness = (int) Math.floor(p2 / 100. * this.domainSize * this.domainSize);
						this.nbConstraints = this.nbConnections * this.nbConstraintsPerConnection + nbIntraAgentConstraints * this.numAgents;
						String subsubDir = "0." + ((Cx < 10)? "0" + Cx : Cx) + "_0." + ((p2 < 10)? "0" + p2 : p2);
						File subdirectory = new File(this.path + File.separator + subDir + File.separator + subsubDir);
						if (!subdirectory.exists()) {
							subdirectory.mkdir();
						}
						for (int i = 0 ; i < this.nbInstances ; i++) {
							this.generateInstance(i);
							this.save(subdirectory.getPath() + File.separator + name + (i + 1) + ".xml");
						}
						Element Class = new Element("class");
						Class.setAttribute("directory", subDir + File.separator + subsubDir);
						Class.setAttribute("Local-constraints-density", "0." + ((p1 < 10)? "0" + p1 : p1));
						Class.setAttribute("ConstraintPerConnection", "0." + ((CpCx < 10)? "0" + CpCx : CpCx));
						Class.setAttribute("Connections", "0." + ((Cx < 10)? "0" + Cx : Cx));
						Class.setAttribute("Tightness", "0." + ((p2 < 10)? "0" + p2 : p2));
						Class.setAttribute("nbInstance", this.nbInstances + "");
						Class.setText(1 + ".." + this.nbInstances);
						configuration.addContent(Class);
					}
				}
			}
		}
		try {
			File conf =
					new File(this.path + File.separator + "ComplexDisCSP_" + this.numAgents + "_" + this.numVarsPerAgent + "_" + this.domainSize
							+ "_" + this.nbInstances + ".xml");
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(confdocument, new FileOutputStream(conf));
		} catch (java.io.IOException e) {}
	}

	/**
	 * @param instance
	 * @param nbI
	 */
	private void generateInstance(int nbI) {
		this.instance = new Element("instance");
		this.document = new Document(instance);
		Random rand = new Random();
		rand.setSeed(System.currentTimeMillis());
		String tuples;
		int PossibleIntraCTs = this.numVarsPerAgent * (this.numVarsPerAgent - 1) / 2;
		int PossibleCxs = this.numAgents * (this.numAgents - 1) / 2;
		int PossibleCpCxs = this.numVarsPerAgent * this.numVarsPerAgent;
		int PossibleNGs = this.domainSize * this.domainSize;
		int[] IntraCTarray = new int[PossibleIntraCTs];
		int[] Cxarray = new int[PossibleCxs];
		int[] CpCxarray = new int[PossibleCpCxs];
		int[] NGarray = new int[PossibleNGs];

		Element presentation = null;
		Element agents = null;
		Element domains = null;
		Element variables = null;
		write(presentation, agents, domains, variables, nbI);

		Element relations = new Element("relations");
		Element constraints = new Element("constraints");
		this.instance.addContent(relations);
		this.instance.addContent(constraints);

		/* Create an array for each possible value pair. */
		PossibleNGs = this.domainSize * this.domainSize;

		NGarray = new int[PossibleNGs];
		/*
		 * Initialize the CTarray. Each entry has one var in the high two bytes, and the other in the low two bytes.
		 */
		int nbConst = 0;
		constraints.setAttribute("nbConstraints", "" + this.nbConstraints);
		relations.setAttribute("nbRelations", "" + this.nbConstraints);

		// generate Intra-Agent constraints
		// **************************************************
		for (int v1 = 0 , i = 0 ; v1 < (this.numVarsPerAgent - 1) ; ++v1)
			for (int v2 = v1 + 1 ; v2 < this.numVarsPerAgent ; ++v2)
				IntraCTarray[i++] = (v1 << 16) | v2;

		for (int ag = 1 ; ag <= this.numAgents ; ag++) {
			for (int c = 0 ; c < this.nbIntraAgentConstraints ; ++c) {
				int r = c + (int) (rand.nextFloat() * (PossibleIntraCTs - c));
				int selectedCT = IntraCTarray[r];
				IntraCTarray[r] = IntraCTarray[c];
				IntraCTarray[c] = selectedCT;
				nbConst++;
				Element constraint = new Element("constraint");
				constraint.setAttribute("name", "C" + nbConst);
				constraint.setAttribute("model", "TKC");
				constraint.setAttribute("arity", "2");
				/* Broadcast the constraint. */
				int vr1 = (int) (IntraCTarray[c] >> 16);
				int vr2 = (int) (IntraCTarray[c] & 0x0000FFFF);
				constraint.setAttribute("scopeAg", "A" + ag);
				constraint.setAttribute("scope", "X" + ag + "." + vr1 + " X" + ag + "." + vr2);
				constraint.setAttribute("reference", "R" + nbConst);
				constraints.addContent(constraint);
				/* Initialize the NGarray. */
				for (int i = 0 ; i < PossibleNGs ; ++i)
					NGarray[i] = i;
				Element relation = new Element("relation");
				relation.setAttribute("name", "R" + nbConst);
				relation.setAttribute("arity", "2");
				relation.setAttribute("nbTuples", "" + this.tightness);
				relation.setAttribute("semantics", "conflits");
				tuples = "";
				/* Select T incompatible pairs. */
				for (int t = 0 ; t < this.tightness ; ++t) {
					/*
					 * Choose a random number between t and PossibleNGs - 1, inclusive.
					 */
					r = t + (int) (rand.nextFloat() * (PossibleNGs - t));
					int selectedNG = NGarray[r];
					NGarray[r] = NGarray[t];
					NGarray[t] = selectedNG;
					/* Broadcast the nogood value pair. */
					int vl1 = (int) (NGarray[t] / this.domainSize);
					int vl2 = (int) (NGarray[t] % this.domainSize);
					tuples = tuples.concat(vl1 + " " + vl2);
					if (t < this.tightness - 1) {
						tuples = tuples.concat("|");
					}
				}
				relation.setText(tuples);
				relations.addContent(relation);
			}
		}

		// **********************************************************************************

		for (int ag1 = 0 , i = 0 ; ag1 < (this.numAgents - 1) ; ++ag1)
			for (int ag2 = ag1 + 1 ; ag2 < this.numAgents ; ++ag2)
				Cxarray[i++] = (ag1 << 16) | ag2;

		/* Select connections. */
		for (int cx = 0 ; cx < this.nbConnections ; ++cx) {
			/* Choose a random number between cx and PossibleCxs - 1, inclusive. */
			int r = cx + (int) (rand.nextFloat() * (PossibleCxs - cx));
			/* Swap elements [c] and [r]. */
			int selectedCx = Cxarray[r];
			Cxarray[r] = Cxarray[cx];
			Cxarray[cx] = selectedCx;
			/* Initialize the NGarray. */
			for (int i = 0 ; i < PossibleCpCxs ; ++i)
				CpCxarray[i] = i;

			for (int cpcx = 0 ; cpcx < this.nbConstraintsPerConnection ; ++cpcx) {
				r = cpcx + (int) (rand.nextFloat() * (PossibleCpCxs - cpcx));
				int selectedCpCx = CpCxarray[r];
				CpCxarray[r] = CpCxarray[cpcx];
				CpCxarray[cpcx] = selectedCpCx;
				nbConst++;
				Element constraint = new Element("constraint");
				constraint.setAttribute("name", "C" + nbConst);
				constraint.setAttribute("model", "TKC");
				constraint.setAttribute("arity", "2");
				/* Broadcast the constraint. */
				int ag1 = (int) (Cxarray[cx] >> 16) + 1;
				int ag2 = (int) (Cxarray[cx] & 0x0000FFFF) + 1;
				int vr1 = (int) (CpCxarray[cpcx] / this.numVarsPerAgent);
				int vr2 = (int) (CpCxarray[cpcx] % this.numVarsPerAgent);
				constraint.setAttribute("scopeAg", "A" + ag1 + " A" + ag2);
				constraint.setAttribute("scope", "X" + ag1 + "." + vr1 + " X" + ag2 + "." + vr2);
				constraint.setAttribute("reference", "R" + nbConst);
				constraints.addContent(constraint);

				/* Initialize the NGarray. */
				for (int i = 0 ; i < PossibleNGs ; ++i)
					NGarray[i] = i;
				Element relation = new Element("relation");
				relation.setAttribute("name", "R" + nbConst);
				relation.setAttribute("arity", "2");
				relation.setAttribute("nbTuples", "" + +this.tightness);
				relation.setAttribute("semantics", "conflits");
				tuples = "";
				/* Select T incompatible pairs. */
				for (int t = 0 ; t < this.tightness ; ++t) {
					/*
					 * Choose a random number between t and PossibleNGs - 1, inclusive.
					 */
					r = t + (int) (rand.nextFloat() * (PossibleNGs - t));
					int selectedNG = NGarray[r];
					NGarray[r] = NGarray[t];
					NGarray[t] = selectedNG;
					/* Broadcast the nogood value pair. */
					int vl1 = (int) (NGarray[t] / this.domainSize);
					int vl2 = (int) (NGarray[t] % this.domainSize);
					tuples = tuples.concat(vl1 + " " + vl2);
					if (t < this.tightness - 1) {
						tuples = tuples.concat("|");
					}
				}
				relation.setText(tuples);
				relations.addContent(relation);
			}
		}
	}

	/**
	 * 
	 * @param str
	 * @return
	 */
	public static boolean isInt(String str) {
		if (str.length() > 0 && str.charAt(0) != '+' && str.charAt(0) != '-' && !(str.charAt(0) >= '0' && str.charAt(0) <= '9'))
			return false;
		for (int i = 1 ; i < str.length() ; i++) {
			char ch = str.charAt(i);
			if (ch < '0' || ch > '9')
				return false;
		}
		return true;
	}

	public static void lunchXP(File file) {// TODO .xml.bz2
	// int nbInstance, connectionDensity, lConstrDensity,
	// constrPerconnectionDensity, instance = 1, nbAgents, nbVarsPerAgent,
	// domainSize;
	// String directory, filePath = fileXP.getParent(), format;
	// int minECCs = aboutDialog.getMinimum();
	// int maxECCs = aboutDialog.getMaximum();
	// String ch = aboutDialog.getTypeOfChannel();
	// String com = aboutDialog.getTypeOfCommunication();
	// int nbOfRun = aboutDialog.getNbOfRuns();
	// String dir = filePath+File.separator+com+"-"+ch;
	// ////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Document document = null;
	// SAXBuilder sxb = new SAXBuilder();
	// try{
	// document = sxb.build(fileXP);
	// } catch(Exception e){
	// }
	// Element root = document.getRootElement();
	// format = root.getAttribute("format").getValue();
	// nbAgents = root.getAttribute(Commands.nbAgents).getIntValue();
	// nbVarsPerAgent = root.getAttribute("nbVarsPerAgents").getIntValue();
	// domainSize = root.getAttribute("domainSize").getIntValue();
	// String name = root.getAttribute("name").getValue();
	// List<Element> children = root.getChildren();
	// ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// for(int p=0 ; p<protocols.length ;p++){ //Protocols
	// Logger allResuls = new Logger(dir, "All_"+protocols[p] +".csv");
	// Logger results = new Logger(dir, protocols[p]+".csv");
	// Iterator<Element> it = children.iterator();
	// boolean header = true;
	// while(it.hasNext()){ //Class Of instance
	// Element child = it.next();
	// directory = child.getAttribute("directory").getValue();
	// lConstrDensity =
	// (int)(child.getAttribute("Local-constraints-density").getDoubleValue()*100);
	// constrPerconnectionDensity =
	// (int)(child.getAttribute("ConstraintPerConnection").getDoubleValue()*100);
	// connectionDensity =
	// (int)(child.getAttribute("Connections").getDoubleValue()*100);
	// nbInstance = child.getAttribute("nbInstance").getIntValue();
	// StringTokenizer st = new StringTokenizer(child.getText());
	// while (st.hasMoreTokens()) {
	// String token = st.nextToken();
	// int position = token.indexOf("..");
	// if (position != -1){
	// instance = Integer.parseInt(token.substring(0, position));
	// }
	// }
	// HashMap<String, Long> stat1 = new HashMap<String, Long>();
	// DisSolverSimulator solver = null;
	// Master master = null;
	// DisProblem disProb = null;
	// for(int i=0; i<nbInstance ; i++, instance++){
	// InstanceParser parser = new InstanceParser();
	// String file = filePath+ File.separator+directory+
	// File.separator+name+instance+".xml";
	// parser.loadInstance(file);
	// parser.parse(false);
	// HashMap<String, Long> stat2 = new HashMap<String, Long>();
	// for(int r=0 ; r<nbOfRun ; r++){
	// master = Protocols.getMaster(protocols[p], format);
	// disProb = new DisProblem(parser.getProblemName(), master, parser);
	// solver = new DisSolverSimulator(disProb, minECCs, maxECCs);
	// solver.solve();
	// HashMap<String, Long> stat = solver.computeStatistics();
	// if(header){
	// header = false;
	// results.printToLogln(Commands.Protocol+","+protocols[p]);
	// results.printToLogln("nbVarsPerAgent,"+nbVarsPerAgent+","+Commands.DOMAIN_SIZE+","+domainSize);
	// results.printToLogln("channel,"+ch+",Communication,"+com+"-"+ch+",minECCs,"+minECCs+",maxECCs,"+maxECCs);
	// results.printToLogln("");
	// results.printToLogln("Connections, Constr-per-connection, local-constr-density,"+Commands.nbAgents+","+solver.statisticsKeysToString(stat)+",% End with Success");
	// allResuls.printToLogln(Commands.Protocol+","+protocols[p]);
	// allResuls.printToLogln("nbVarsPerAgent,"+nbVarsPerAgent+",domainSize,"+domainSize);
	// allResuls.printToLogln("channel,"+ch+",Communication,"+com+"-"+ch+",minECCs,"+minECCs+",maxECCs,"+maxECCs);
	// allResuls.printToLogln("");
	// allResuls.printToLogln("instance,problem,Connections,Constr-per-connection,local-constr-density,"+Commands.nbAgents+","+solver.statisticsKeysToString(stat)+
	// ",State");
	// }
	// stat2 = Utility.merge(stat, stat2);
	// allResuls.printToLogln((i+1)+","+(r+1)+",0."+connectionDensity+",0."+constrPerconnectionDensity+",0."+
	// lConstrDensity+","+nbAgents+","+ Utility.statisticsToString(stat,
	// minECCs)+","+ master.getEndState());
	// disProb.clear();
	// disProb = null;
	// }
	// Utility.average(stat2, nbOfRun);
	// stat1 = Utility.merge(stat2, stat1);
	// parser.clear();
	// parser = null;
	// }
	// Utility.average(stat1, nbInstance);
	// results.printToLogln("0."+connectionDensity+",0."+constrPerconnectionDensity+",0."+
	// lConstrDensity+","+nbAgents+","+ Utility.statisticsToString(stat1,
	// minECCs));
	// }
	// console.replaceSelection("The protocol: "+protocols[p]+" was experimented with success \n");
	// console.replaceSelection("Results of "+protocols[p]+" are in "+filePath+
	// File.separator+results.getPath()+"\n");
	// results.close();
	// allResuls.close();
	// }
	// printResults(dir+ File.separator, protocols, domainSize, minECCs,
	// children.size());
	// /////////////////////////////////////////////////////////////////////////////////////////////////////////
	}

	public static void printResults(String dir, String[] protocols, int domainSize, int minECCs, int nbLine) {
		Logger result = new Logger(dir, "ALLRESULTS.csv");
		BufferedReader[] br = new BufferedReader[protocols.length];
		String line = null;
		String[][] proLine = new String[protocols.length][4];
		try {
			for (int p = 0 ; p < protocols.length ; p++) {
				br[p] = new BufferedReader(new FileReader(dir + protocols[p] + ".csv"));
			}
			// ignore first line
			for (int p = 0 ; p < protocols.length ; p++) {
				br[p].readLine();
			}
			for (int k = 0 ; k < 2 ; k++) {
				line = br[0].readLine();
				result.printToLogln(line);
				for (int p = 1 ; p < protocols.length ; p++) {
					br[p].readLine();
				}
			}// ignore two line
			for (int p = 0 ; p < protocols.length ; p++) {
				br[p].readLine();
				br[p].readLine();
			}
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		line = ",,,," + ((minECCs == 0)? Commands.NCCCs : Commands.NCCCs);
		for (int p = 0 ; p < protocols.length ; p++) {
			line += ",";
		}
		line += Commands.NCSteps;
		for (int p = 0 ; p < protocols.length ; p++) {
			line += ",";
		}
		line += Commands.MSGs;
		for (int p = 0 ; p < protocols.length ; p++) {
			line += ",";
		}
		line += Commands.SOLVABLE;
		for (int p = 0 ; p < protocols.length ; p++) {
			line += ",";
		}
		result.printToLogln(line);
		line = "Connections,Constr-per-connection,local-constr-density," + NB_AGENTS;
		for (int k = 0 ; k < 4 ; k++) {
			for (int p = 0 ; p < protocols.length ; p++) {
				line += "," + protocols[p];
			}
		}
		result.printToLogln(line);
		try {
			for (int l = 0 ; l < nbLine ; l++) {
				String[] str = null;
				for (int p = 0 ; p < protocols.length ; p++) {
					line = br[p].readLine();
					str = line.split(",");
					for (int k = 0 ; k < 4 ; k++) {
						proLine[p][k] = str[k + 4];
					}
				}
				line = str[0] + "," + str[1] + "," + str[2] + "," + str[3];
				for (int k = 0 ; k < 4 ; k++) {
					for (int p = 0 ; p < protocols.length ; p++) {
						line += "," + proLine[p][k];
					}
				}
				result.printToLogln(line);
			}
			for (int p = 0 ; p < protocols.length ; p++) {
				br[p].close();
			}
			result.close();
			return;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws IOException {
		int minCx = 20, maxCx = 90, stepCx = 20; // number of connections
		int minCpCx = 50, maxCpCx = 50, stepCpCx = 10; // number of constraints
		// per connection
		int minP1 = 50, maxP1 = 60, stepP1 = 10; // number of local constraint
		// per agent
		int minP2 = 10, maxP2 = 90, stepP2 = 10; // Tightness (number of
		// infeasible/feasible
		// tuples)
		File file = new File("ComplexTest");
		RBCDisCSPGenerator g = new RBCDisCSPGenerator(file, 5, 4, 3, 5);
		g.setConnection(minCx, maxCx, stepCx);
		g.setConstraintsPerConnection(minCpCx, maxCpCx, stepCpCx);
		g.setLocalConstraintDensity(minP1, maxP1, stepP1);
		g.setTightness(minP2, maxP2, stepP2);
		g.generate();
		System.out.println("Problems were generated Successfully in " + file);
	}

	/* (non-Javadoc)
	 * @see gui.GuiSimulator#lunch()
	 */
	@Override
	public void lunch() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see gui.GuiSimulator#resumeWithoutSolver()
	 */
	@Override
	public void resumeWithoutSolver() {
		// TODO Auto-generated method stub
		System.err.println("I have to resume my experiments such that my solver is null");
	}

}
