package kernel;

import java.util.HashMap;

import util.IVarIdentifier;

/**
 * A class for representing the identifiers of variables.
 * 
 * 
 * @author Redouane Ezzahir
 * 
 */
public final class VarIdentifier implements IVarIdentifier, Cloneable, Comparable<IVarIdentifier> {
	/**
	 * 
	 */
	private static final long serialVersionUID = -4527543357838318955L;
	/**
	 * Identifier of the agent owning the variable with the identifier.
	 */
	private final int agentId;
	/**
	 * Index of this variable in its local problem.
	 */
	private final int varIndex;

	// public VarIdentifier(){ }
	/**
	 * Constructs a variable identifier
	 * 
	 * @param agId
	 *            the Id of the agent owning the variable with the identifier
	 * @param varIndex
	 *            the index in the local problem of the variable with the identifier
	 */
	public VarIdentifier(int agId, int varIndex) {
		this.agentId = agId;
		this.varIndex = varIndex;
	}

	public VarIdentifier(int agId) {
		this(agId, 0);
	}

	/**
	 * Returns the index in the local problem of the variable with the identifier
	 */

	public int getVarIndex() {
		return this.varIndex;
	}

	/**
	 * Returns the ID of the agent owning the variable with the identifier.
	 */
	public int getAgentId() {
		return this.agentId;
	}

	/**
	 * Returns a cloned object of the variable identifier
	 */
	public VarIdentifier clone() {
		return new VarIdentifier(this.agentId, this.varIndex);
	}

	/**
	 * Checks whether the identifier is equals to the given object
	 */
	public boolean equals(Object o) {
		VarIdentifier varId = (VarIdentifier) o;
		if (varId == null)
			return false;
		return (this.agentId == varId.agentId) && (this.varIndex == varId.varIndex);
	}

	public int hashCode() {
		return this.agentId * 1000 + this.varIndex;
	}

	/**
	 * Compare two variable identifiers lexicographically.
	 * 
	 * @return this.agentId - v.agentId if different from zero this.varIndex - v. varIndex otherwise
	 */
	@Override
	public int compareTo(IVarIdentifier v) {
		if (v != null) {
			int d = this.getAgentId() - v.getAgentId();
			if (d == 0)
				d = this.getVarIndex() - v.getVarIndex();
			return d;
		}
		return Integer.MIN_VALUE;
	}

	/**
	 * Returns a string representing this object
	 */
	public String toString() {
		return this.agentId + "." + this.varIndex;
	}

	/**
	 * Make a VarIdentifier from the given string
	 * 
	 * @param str
	 *            - the string to construct the identifier from; Example "5.1"
	 * @return A new VarIdentifier
	 */
	public synchronized static VarIdentifier fromString(String str) {
		str = str.trim();
		VarIdentifier vid = null;
		char[] sa = str.toCharArray();
		int i = 0;
		String adID = new String();
		String vId = new String();
		for ( ; i < sa.length ; i++) {
			if (sa[i] == '.')
				break;
			adID += String.valueOf(sa[i]);
		}
		i++;
		for ( ; i < sa.length ; i++) {
			vId += String.valueOf(sa[i]);
		}
		int agentId = new Integer(adID);
		int varIndex = new Integer(vId);
		vid = new VarIdentifier(agentId, varIndex);

		return vid;
	}

	public static void main(String args[]) {
		String[] vi = {"1.0", "2.0", "3.0", "4.0", "14.0", "5.0", "6.0", "7.0", "8.0", "9.0", "10.0", "11.0", "12.0", "13.0", "15.0"};

		HashMap<VarIdentifier, Integer> map = new java.util.HashMap<VarIdentifier, Integer>();
		for (int i = 0 ; i < vi.length ; i++) {
			VarIdentifier v = VarIdentifier.fromString(vi[i]);

			map.put(v, v.hashCode());
		}
		System.out.println(map + " " + map.containsKey(new VarIdentifier(15)));
	}

}
