package kernel.com;

import java.io.Serializable;
import java.util.Hashtable;

/**
 * This class models a message that will be exchanged between agents. Message contains a set of attributes (from, to, type, ...) and a "content".
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */

public class Message implements Cloneable, Serializable {

	private static final long serialVersionUID = 5725211850619445129L;
	/**
	 * The type of the message
	 */
	private final String msgType;
	/**
	 * Denotes the identity of the sender of the message
	 */
	private int from;
	/**
	 * Denotes the identity of the intended recipients of the message
	 */
	private int to;
	/**
	 * The contents of the message
	 */
	private Hashtable<String, Object> contents;
	/**
	 * message Id that allow to give the causal order of messages sent
	 */
	private long msgId = 0;

	private long totalMsg = 0;
	/**
	 * A field to attach the non concurrent computation time
	 */
	private long NCCT = -1;
//	/**  A field to attach the number of constraint checks */
//	private long CCs = -1;

	/**
	 * Constructs a new message within the given type
	 * 
	 * @param type
	 *            : a string repressing the type of the message to make
	 */

	public Message(String type) {
		msgType = type;
	}

	/**
	 * Returns the type of the message
	 */

	public String getType() {
		return this.msgType;
	}

	/**
	 * Set the recipient ID.
	 * 
	 * @param to
	 *            Id of the agent recipient
	 */

	public void setReceiverId(int to) {
		this.to = to;
	}

	/**
	 * Returns the ID of the recipient
	 */
	public int getReceiverId() {
		return to;
	}

	/**
	 * Set the sender ID
	 * 
	 * @param from
	 *            : ID of the sender
	 */
	public void setSenderId(int from) {
		this.from = from;
	}

	/**
	 * Returns the sender ID
	 */
	public int getSenderId() {
		return this.from;
	}

	/**
	 * Returns the message ID
	 */
	public long getMsgId() {
		return msgId;
	}

	/**
	 * Set the message ID to the given value
	 */
	public void setMsgId(long msgId) {
		this.msgId = msgId;
	}

	/**
	 * Adds a new data to the contents of the message It Maps the specified <code>key</code> to the specified <code>value</code> in contents
	 * hashtable.
	 * 
	 * @param key
	 *            the hashtable key
	 * @param value
	 *            the data
	 */
	public void addContents(String key, Serializable value) {
		if (contents == null)
			contents = new Hashtable<String, Object>();
		contents.put(key, value);
	}

	/**
	 * Returns the number of non concurrent computation time
	 */
	public long getNCCT() {
		return NCCT;
	}

	/**
	 * Set the number of non concurrent computation time to the given value
	 */
	public void setNCCT(long ncct) {
		this.NCCT = ncct;
	}

//	/**
//	 * Returns the number of constraint checks
//	 */
//	public long getCCs() {
//		return this.CCs;
//	}
//
//	/**
//	 * Set the number of constraint checks to the given value
//	 */
//	public void setCCs(long CCs) {
//		this.CCs  = CCs;
//	}

	/**
	 * Returns the data to which the specified key is mapped, or {@code null} if the contents of this message contains no mapping for the key.
	 * 
	 * @param key
	 *            key the key whose associated data is to be returned
	 * @return the data to which the specified key is mapped, or {@code null} the contents of this message contains no mapping for the key
	 */
	public Object get(String key) {
		if (contents != null)
			return contents.get(key);
		return null;
	}

	/**
	 * Returns <tt>true</tt> if the contents of this message contains a mapping for the specified key. More formally, returns <tt>true</tt> if and
	 * only if the contents of this message contains a mapping for a key <tt>key</tt> such that <tt>(key==null ? k==null : key.equals(k))</tt>.
	 */
	public boolean contains(String key) {
		return this.get(key) != null;
	}

	/**
	 * Returns a clone of the message
	 */
	@SuppressWarnings("unchecked")
	public Message clone() {
		Message msg = new Message(msgType);
		msg.from = this.from;
		msg.to = this.to;
		msg.msgId = this.msgId;
		msg.NCCT = this.NCCT;
		msg.contents = (Hashtable<String, Object>) this.contents.clone();
		return msg;
	}

	/**
	 * Returns a string representation of this message
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("message <Id:" + this.msgId + " Type :" + this.msgType + "> from " + from + " to " + to + " : ");
		sb.append("[ ");
		if (this.contents != null) {
			for (String key : this.contents.keySet()) {
				sb.append("" + key + " : " + this.contents.get(key) + " ");
			}
		}
		sb.append("]");
		return sb.toString();
	}
	
	public long getSize(){
		long size = 0;
		if (this.contents != null) {
			for (String key : this.contents.keySet()) {
				size += this.contents.get(key).toString().length();
			}
		} 
		return size;
	}

	/**
	 * An implementation of messages comparator. it is used to store message in a stored set
	 */
	public static class Comparator implements java.util.Comparator<Message> {
		public int compare(Message msg1, Message msg2) {
			long def1 = (msg1.getNCCT() - msg2.getNCCT());
			if (def1 != 0)
				return (int) def1;
			long def2 = msg1.msgId - msg2.msgId;
			if (def2 != 0)
				return (int) def2;
			return msg1.from - msg2.from;
		}
	}

	public long getTotatNumMessages() {
		return totalMsg;
	}

	/**
	 * Set the number of non concurrent computation time to the given value
	 */
	public void setTotalNumMessage(long totalmsg) {
		this.totalMsg = totalmsg;
	}

}
