package kernel.constraints.integer;

import java.util.BitSet;

public class CouplesTable extends ConsistencyRelation implements BinRelation {

	/**
	 * matrix of consistency/inconsistency
	 */
	protected BitSet table;

	/**
	 * first value of x
	 */
	protected int offset1;

	/**
	 * first value of y
	 */
	protected int offset2;

	/**
	 * size of the initial domain of x
	 */
	protected int n2;
	/**
	 * A filed stores whether the relation is transposed or not.
	 */
	protected boolean transpose = false;
	
	int[] nbSupport = null;
	
	public CouplesTable() {}

	public CouplesTable(boolean feas, int offset1, int offset2, int n1, int n2) {
		this.offset1 = offset1;
		this.offset2 = offset2;
		this.n2 = n2;
		this.table = new BitSet(n1 * n2);
		this.feasible = feas;
		this.nbSupport = new int[n2];
		for(int i=0 ; i<this.nbSupport.length ; i++){
			if(!this.feasible){
				this.nbSupport[i] = n1;
			} else {
				this.nbSupport[i] = 0;
			}
		}
	}

	/**
	 * Compute the opposite relation by "reusing" the table of consistency
	 * 
	 * @return the opposite relation
	 */
	public ConsistencyRelation getOpposite() {
		CouplesTable t = new CouplesTable();
		t.feasible = !feasible;
		t.table = table;
		t.offset1 = offset1;
		t.offset2 = offset2;
		t.n2 = n2;
		t.nbSupport = nbSupport;
		return t;
	}

	/**
	 * Adds tuple to the consistency table
	 * 
	 * @param x A value of the first variable
	 * @param y A value of the second variable
	 */
	public void setCouple(int x, int y) {
		if(this.feasible){
			if (this.transpose){
				++this.nbSupport[y - offset1];
			} else { 
				++this.nbSupport[x - offset1];
			}
		} else {
			if (this.transpose){
				--this.nbSupport[y - offset1];
			} else { 
				--this.nbSupport[x - offset1];
			}
		}
		table.set((x - offset1) * n2 + y - offset2);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.integer.constraints.BinRelation#isConsistent(int, int)
	 */

	public boolean isConsistent(int x, int y) {
		if (this.transpose)
			return table.get((y - offset1) * n2 + x - offset2) == feasible;
		return table.get((x - offset1) * n2 + y - offset2) == feasible;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.integer.constraints.BinRelation#checkCouple(int, int)
	 */
	public boolean checkCouple(int x, int y) {
		if (this.transpose)
			return table.get((y - offset1) * n2 + x - offset2);
		return table.get((x - offset1) * n2 + y - offset2);
	}

	public String toString() {
		String s = "";
		for (int i = this.table.nextSetBit(0) ; i > -1 ; i = this.table.nextSetBit(i + 1)) {
			int y = i % n2 + offset2;
			int x = (i - y + offset2) / n2 + this.offset1;
			s += "(" + x + "," + y + ") ";
		}
		s += "\n { ";
		for(int x=0 ; x<this.nbSupport.length ; x++){
			int value = (x + this.offset1);
			s += "(" + value + "," + this.nbSupport[x] + ") ";
		}
		s += "}";
		s += "\nfeasible : "+this.feasible+"";
		s += "\ntranspose: "+this.transpose+"";
		return "[" + s + "]";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.integer.constraints.ConsistencyRelation#transpose()
	 */
	@Override
	public void transpose() {
		this.transpose = true;
	}

	/* (non-Javadoc)
	 * @see kernel.constraints.integer.BinRelation#getNumberOfSupport(int)
	 */
	@Override
	public int getNumberOfSupports(int value) {
		if(feasible){
			return this.nbSupport[value - offset1];
		} else {
			return this.nbSupport[value - offset1];
		}
	}

	/* (non-Javadoc)
	 * @see kernel.constraints.integer.BinRelation#getNumberOfSupport(int)
	 */
	@Override
	public int getNumberOfConflicts(int value) {
		if(feasible){
			return this.n2 - this.nbSupport[value - offset1];
		} else {
			return this.n2 - this.nbSupport[value - offset1];
		}
	}

}
