/*****************************************************************************************

 *****************************************************************************************/
package kernel.order;

import java.util.Comparator;
import java.util.HashSet;
import java.util.Vector;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.AgentObserver;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.vars.integer.IntExternalVar;
import simulation.metric.MetricListener;
import util.Commands;
import util.IVarIdentifier;
import util.IntArray;
import util.graph.Graph;

/**
 * A Class to store and manage the order of variables/agents. It is used as comparator of variables. The guarantee that the order of variables in the
 * {@link kernel.AgentView} and the instantiations
 * 
 * @see util.Instantiation is the same as the current order.
 */

public class AgentOrdering extends MessageHandler implements Comparator<IVarIdentifier> {
	/**
	 * structure containing all available agents orderings in the plate-form
	 */
	public static final String[] ORDERINGS = {Commands.LexicoAO, Commands.RandomAO, Commands.MinDomDegAO, Commands.MaxDegPlusMinDomAO, Commands.TreeAO};

	protected Graph<Integer> graph = null;
	/**
	 * An array stores the current order. The current order is given by the order of IDs in the array. ID that is in position 0 is the ID of the agent
	 * with the highest priority.
	 */
	protected IntArray agentsOrder = new IntArray();
	/**
	 * A field indicates whether the order is directed or not. ABT uses a directed order but DBA do not uses a directed order
	 */
	protected boolean directed = false;
	/**
	 * A field indicates whether the order is received from the Master or not; NB. Agents can delegate to the Master to construct agents ordering in a
	 * centralized way.
	 */

	protected boolean rcvOrder = false;

	/**
	 * Constructs an agent ordering object
	 * 
	 * @param agent
	 *            the owner of this object
	 */
	public AgentOrdering(AbstractAgent agent) {
		super(agent, Commands.AO);
	}

	public void initialize() {}

	/**
	 * Set the agent order to the given order
	 * 
	 * @param newOrder
	 *            the newest agent order to set
	 */
	public boolean setNewOrder(IntArray newOrder) {
		if (this.agentsOrder.equals(newOrder))
			return false;
		if (this.myAgent.agentView != null) {
			HashSet<IntExternalVar> view = this.myAgent.agentView.getSetOfExternalVars();
			this.agentsOrder = newOrder;
			this.myAgent.agentView.refresh(view);
		} else
			this.agentsOrder = newOrder;
		return true;
	}

	/**
	 * Returns the current order
	 * 
	 * @return the value of agentsOrder
	 */
	public IntArray getOrder() {
		return this.agentsOrder.clone();
	}

	/**
	 * Checks whether the order is directed or not.
	 */
	public boolean isDirectedOrder() {
		return this.directed;
	}

	/**
	 * Marks that the order is directed
	 */
	public void setDirectedOrder() {
		this.directed = true;
	}

	/**
	 * Returns the rank of the agent with the given ID in the current order
	 */
	public int getRankOf(int agId) {
		return this.agentsOrder.indexOf(agId);
	}

	/**
	 * Returns the ID of the agent of the given rank.
	 */
	public int getAgentOfRank(int rank) {
		// assert(rank < this.agentsOrder.size()) ;
		return this.agentsOrder.get(rank);
	}

	/**
	 *Returns an array that represents agents lower than the agent of the given ID
	 */
	public IntArray getAgentsLowerThan(int agId) {
		IntArray lw = new IntArray();
		int rank = this.getRankOf(agId);
		for (int i = rank + 1 ; i < this.agentsOrder.size() ; i++)
			lw.add(this.agentsOrder.get(i));
		return lw;
	}

	/**
	 *Returns an array that represents agents higher than the agent of the given ID
	 */
	public IntArray getAgentsHigherThan(int agId) {
		IntArray higherAgents = new IntArray();
		int rank = this.getRankOf(agId);
		for (int i = 0 ; i < rank ; i++) {
			higherAgents.add(this.agentsOrder.get(i));
		}
		return higherAgents;
	}

	/**
	 *Returns an array that represents agents who are between the agents of the given IDs
	 */
	public IntArray getAgentsBetween(int agId1, int agId2) {
		IntArray res = new IntArray();
		int minRank = Math.min(this.getRankOf(agId1), this.getRankOf(agId2));
		int maxRank = Math.max(this.getRankOf(agId1), this.getRankOf(agId2));
		for (int i = minRank + 1 ; i < maxRank ; i++) {
			res.add(this.agentsOrder.get(i));
		}
		return res;
	}

	/**
	 * Checks whether the agent with the given ID is lower than the agent owning this object or not.
	 * 
	 */

	public boolean isLower(int agId) {
		return this.getLowerAgents().contains(agId);
	}

	/**
	 * Checks whether the agent with the given ID is higher than the agent owning this object or not.
	 * 
	 */
	public boolean isHigher(int agId) {
		return this.getHigherAgents().contains(agId);
	}

	/**
	 * Returns an array that represents agents lower than the agent owning this object.
	 * 
	 * @see #getAgentsLowerThan(int)
	 */
	public IntArray getLowerAgents() {
		return this.getAgentsLowerThan(this.myAgent.agentId);
	}

	/**
	 * Returns an array that represents agents higher than the agent owning this object.
	 * 
	 * @see #getAgentsHigherThan(int)
	 */

	public IntArray getHigherAgents() {
		return this.getAgentsHigherThan(this.myAgent.agentId);
	}

	/**
	 * Returns the ID of the agent that follows the agent with the given ID in the agents order.
	 */
	public int getNextOf(int agId) {
		int rank = this.getRankOf(agId) + 1;
		if (rank < this.agentsOrder.size())
			return this.agentsOrder.get(rank);
		else
			return -1;
	}

	public int getPreviousOf(int agId) {
		int rank = this.getRankOf(agId) - 1;
		if (rank > -1)
			return this.agentsOrder.get(rank);
		else
			return -1;
	}

	/**
	 * Compares the order of two agents Returns : getRankOf(agId1) - getRankOf(agId0)
	 * 
	 * @return >0 if agId0 is higher (of high priority) than agId1 <0 if agId0 is lower (of low priority) than agId1 0 if agId0==agId1
	 * @see #getRankOf(int)
	 */
	public int compareOrder(int agId0, int agId1) {
		return (this.getRankOf(agId1) - this.getRankOf(agId0));
	}

	/**
	 * Chooses and returns the lower agent between the given agents with the given IDs
	 */
	public int chooseLower(int agId0, int agId1) {
		if (this.compareOrder(agId0, agId1) < 0)
			return agId0;
		return agId1;
	}

	/**
	 * Chooses and returns the higher agent between the given agents with the given IDs
	 */
	public int chooseHigher(int agId0, int agId1) {
		if (this.compareOrder(agId0, agId1) > 0)
			return agId0;
		return agId1;
	}

	/**
	 * The default agent ordering uses a centralized algorithms that is running by the master. This method is called to send agent complexity to the
	 * master and block waiting for order.
	 * 
	 * @throws StopException
	 */
	public void startDisAO() throws StopException {
		this.sendComplexity();
		this.waitOrder();
	}

	/**
	 * Blocking the agent owning this object in waiting until the order is received.
	 * 
	 * @throws StopException
	 * @throws
	 */
	public void waitOrder() throws StopException {
		while (!rcvOrder) {
			try {
				this.myAgent.blockingReceiveMessage(Commands.AO, 0);
			} catch (InterruptedException e) {
				e.printStackTrace();
				System.exit(-1);
			}
		}
	}

	/**
	 * {@inheritDoc} handles a message ORDER
	 */
	@SuppressWarnings("unchecked")
	public void processMessage(Message msg) throws StopException {
		if (msg.getType().equals(Commands.AO)) {
			processOrder(msg);
		}
		if (msg.contains(Commands.METRIC)) {
			Vector<MetricListener> metrics = (Vector<MetricListener>) msg.get(Commands.METRIC);
			for (MetricListener metric : metrics) {
				this.myAgent.addObserver((AgentObserver) metric);
			}
		}

	}

	public void processOrder(Message msg) {
		this.agentsOrder = (IntArray) msg.get(Commands.ORDER);
		this.graph = Graph.fromMsg((String) msg.get(Commands.GRAPH));
		this.rcvOrder = true;
		// this.myAgent.log("process Order :" + msg.toString(), Logger.LEVEL_TRACE_MESSAGE);

	}

	/**
	 * Called to send complexity to the master when a centralized agent ordering is used.
	 * 
	 * @see AgentComplexity
	 */
	public void sendComplexity() {
		AgentComplexity cmpx = new AgentComplexity();
		cmpx.setMaxDomain(this.myAgent.getMaxDomainSize());
		cmpx.setNumberConstraints(this.myAgent.constraintHandler.getNumConstraints());
		cmpx.setNumberLocalVars(this.myAgent.getNbVars());
		cmpx.setNeighbors(this.myAgent.communicator.getNeighbors());
		cmpx.setLocalAgents(this.myAgent.localAgents);
		Message msg = this.createMessage();
		msg.addContents(Commands.AGENT_COMPLEXITY, cmpx);
		msg.setReceiverId(AbstractMaster.MasterId);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * Returns the rank of the agent owning this object.
	 */
	public int getMyRank() {
		return this.getRankOf(this.myAgent.getAgentId());
	}

	/**
	 * Return the number of agents (i.e. the size of {@link #agentsOrder}
	 */
	public int getNumAgents() {
		return this.agentsOrder.size();
	}

	/**
	 * Compares the order of two variables with the given identifiers. when they are belong to different agents it returns the result of the
	 * comparison of the order of their agents. When the two variables are belong to the same agent the returned value is difference between the
	 * indices of two variables in the local problem.
	 * 
	 * @see #compareOrder(int, int)
	 */
	public int compare(IVarIdentifier var1, IVarIdentifier var2) {
		int comp = this.compareOrder(var2.getAgentId(), var1.getAgentId());
		return (comp != 0)? comp : var1.compareTo(var2);
	}

	public static CentralizedAO getCentralizedAO(String orderType) {
		if (orderType.equals(Commands.LexicoAO)) { return new LexicographicAO(); }
		if (orderType.equals(Commands.RandomAO)) { return new RandomAO(); }
		if (orderType.equals(Commands.MinDomDegAO)) { return new MinDomDegAO(); }
		if (orderType.equals(Commands.MaxDegPlusMinDomAO)) { return new MaxDegPlusMinDomAO(); }
		if (orderType.equals(Commands.TreeAO)) { return new TreeAO(); }
		throw new Error("Unknown type of order : " + orderType);

	}

	public Iterable<Integer> getNeighborsOf(int agId) {
		return graph.getNeighbors(agId);
	}

	/**
	 * Return names of available agent ordering
	 */
	public static String[] getOrderTypes() {
		return ORDERINGS;
	}

	public Graph<Integer> getAgentsGraph() {
		return this.graph;
	}

	/**
	 * 
	 * @param agentIds
	 * @return
	 */
	public int getLowerAgent(IntArray agentIds) {
		for (int agId : agentIds) {
			IntArray lowerAgents = this.getAgentsLowerThan(agId);
			if (IntArray.intersection(agentIds, lowerAgents).size() == 0) { return agId; }
		}
		return -1;
	}

	/**
	 * 
	 * @param agentIds
	 * @return
	 */
	public int getHigherAgent(IntArray agentIds) {
		for (int agId : agentIds) {
			IntArray higherAgents = this.getAgentsHigherThan(agId);
			if (IntArray.intersection(agentIds, higherAgents).size() == 0) { return agId; }
		}
		return -1;
	}

	/**
	 * @param newOrdering
	 */
	public void updateOrder(IntArray agentIds) {
		for (int agId : this.agentsOrder) {
			if (!agentIds.contains(agId)) agentIds.add(agId);
		}
		this.setNewOrder(agentIds);
	}

}
