package kernel.order;

import java.util.HashMap;
import java.util.Vector;

import kernel.AbstractAgent;
import kernel.VarIdentifier;
import kernel.com.Message;
import util.IntArray;
import util.Tools;
import util.graph.PriorityTree;

public class TreeDisAO extends AgentOrdering {

	/* my parent */
	protected int parentId;
	/* my children */
	protected IntArray childrenIds;
	/* agents graph on tree structure */
	protected PriorityTree orderTree;
	/* agents neighborsIds */
	HashMap<Integer, IntArray> neighborsMap;
	/**
	 * ancestors and descendants : neighbors union children
	 */
	protected IntArray descendants; // my sons, their sons, ...
	protected IntArray ancestors; // my father, his father, ...

	private IntArray pseudoDecendants = null;
	private IntArray pseudoAncestors;

	public TreeDisAO(AbstractAgent agent) {
		super(agent);
	}

	public void processOrder(Message msg) {
		super.processOrder(msg);
		this.parentId = Integer.parseInt((String) msg.get("Parent-Id"));
		this.childrenIds = (IntArray) msg.get("Children-Ids");
		this.orderTree = PriorityTree.fromString((String) msg.get("Priority-tree"));
		this.neighborsMap = Tools.mapIntegerIntArrayfromString((String) msg.get("Neighbors"));
		this.rcvOrder = true;
		// this.myAgent.log("Message Order arrived :" + msg.toString(), Logger.LEVEL_TRACE_MESSAGE);
	}

	public void initialize() {
		for (int chId : this.childrenIds)
			if (!this.myAgent.communicator.isNeighbor(chId)) {
				this.addLink(new VarIdentifier(chId));
			}
		// ----------------------------------------------------------------------------//
		if (parentId!=-1 && !this.myAgent.communicator.isNeighbor(parentId)) {
			this.addLink(new VarIdentifier(parentId));
		}
		// ----------------------------------------------------------------------------//
		this.descendants = this.orderTree.getDecendants(this.myAgent.getAgentId());
		// ----------------------------------------------------------------------------//
		this.ancestors = this.orderTree.getAncestors(this.myAgent.getAgentId());
		this.pseudoAncestors = this.getPseudoAncestors(this.myAgent.getAgentId());
		this.pseudoDecendants = this.getPseudoDescendants(this.myAgent.getAgentId());
	}

	private void addLink(VarIdentifier varId) {
		this.myAgent.makeExternalVar(varId);
		this.myAgent.communicator.addNeighbor(varId.getAgentId());
	}

	public boolean isMyChild(int agId) {
		return this.childrenIds.contains(agId);
	}

	public boolean idMyParent(int agId) {
		return this.parentId == agId;
	}

	/**
	 * @return IntArray
	 */
	public IntArray getDescendants() {
		return this.descendants;
	}

	/**
	 * @return IntArray
	 */
	public IntArray getAncestors() {
		return this.ancestors;
	}

	public IntArray getChildrenIds() {
		return this.childrenIds;
	}

	public int getParentId() {
		return this.parentId;
	}

	public IntArray getPseudoAncestors() {
		return this.pseudoAncestors;
	}

	/**
	 * return the sub-set of ancestors of the given agent (including its parent) that it or one of its descendants is involved in constraints with
	 * 
	 * @param agId
	 * @return
	 */

	public IntArray getPseudoAncestors(int agId) {
		IntArray res = new IntArray();
		if (this.orderTree.getParent(agId) == null)
			return res;
		res.add(this.orderTree.getParent(agId).getRootId());
		IntArray desc = this.orderTree.getDecendants(agId);
		IntArray ances = this.orderTree.getAncestors(agId);
		for (int a : ances) {
			if (this.neighborsMap.get(agId).contains(a)) {
				res.add(a);
			} else {
				for (int d : desc) {
					if (this.neighborsMap.get(a).contains(d)) {
						res.add(a);
					}
				}
			}
		}
		return res;
	}

	/**
	 * return the sub-set of descendants of the given agent (including its children) that it or one of its ancestors is involved in constraints with
	 * 
	 * @param agId
	 * @return
	 */

	public IntArray getPseudoDescendants(int agId) {
		IntArray res = new IntArray();
		if (this.orderTree.getDecendants(agId).size() == 0)
			return res;
		IntArray desc = this.orderTree.getDecendants(agId);
		IntArray ances = this.orderTree.getAncestors(agId);
		for (int d : desc) {
			if (this.neighborsMap.get(agId).contains(d)) {
				res.add(d);
			} else {
				for (int a : ances) {
					if (this.neighborsMap.get(a).contains(d)) {
						res.add(d);
					}
				}
			}
		}
		res = IntArray.union(res, this.getChildrenIds(agId));
		return res;
	}

	public IntArray getChildrenIds(int agId) {
		IntArray res = new IntArray();
		if (this.orderTree.getSubTree(agId) != null) {
			Vector<PriorityTree> ch = this.orderTree.getSubTree(agId).getChildren();
			for (PriorityTree c : ch) {
				res.add(c.getRootId());
			}
		}
		return res;
	}

	/**
	 * 
	 * @param agId
	 * @return
	 */

	public IntArray getAncestorsBeforFirstNeighbor(int agId) {
		IntArray res = new IntArray();
		if (this.orderTree.getParent(agId) == null)
			return res;
		IntArray anc = this.orderTree.getAncestors(agId);
		IntArray order = this.myAgent.agentOrder.getAgentsHigherThan(agId);
		for (int i = 0 ; i < order.size() && i < anc.size() ; i++) {
			int id = order.get(i);
			if (anc.contains(id)) {
				if (!this.neighborsMap.get(agId).contains(id)) {
					res.add(id);
				}
			}
		}
		return res;
	}

	public IntArray getDescendants(int childId) {
		return this.orderTree.getDecendants(childId);
	}

	public PriorityTree getPriorityOrderTree() {
		return this.orderTree;
	}

	public IntArray getPseudoDescendants() {
		return this.pseudoDecendants;
	}

	public int getMyLevel() {
		return this.getAncestors().size();
	}

}
