package simple.dcsp.afcfamily.afcng.domDeg;

import java.util.HashMap;

import kernel.AbstractAgent;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;

import simple.dcsp.afcfamily.afcng.DynamicAFCngAgent;
import util.Commands;
import util.Instantiation;
import util.IntArray;

public class AFCngDomDegAgent extends DynamicAFCngAgent {

	protected HashMap<Integer, Long> counters;
	protected HashMap<Integer, Integer> domainSizes;
	protected DomMsgHandler domSizeH;

	public static final int MIN_DOM = 0;
	public static final int DOM_DEG = 1;
	public static final int DOM_FUTUR_DEG = 2;
	protected int heuristic = DOM_FUTUR_DEG; // static or dynamic heuristic

	/**
	 * 
	 * @param agentId
	 * @param agName
	 * @param description
	 * @param heuristic
	 */
	public AFCngDomDegAgent(int agentId, String agName, String description, int heuristic) {
		super(agentId, agName, description);
		this.domSizeH = new DomMsgHandler(this);
		this.domainSizes = new HashMap<Integer, Integer>();
		this.counters = new HashMap<Integer, Long>();
		this.heuristic = heuristic;
		for (int agId : this.getOrder()) {
			domainSizes.put(agId, Integer.MAX_VALUE);
			counters.put(agId, 0L);
		}
	}
	/**
	 * 
	 * @param agentId
	 * @param agName
	 * @param description
	 */
	public AFCngDomDegAgent(int agentId, String agName, String description) {
		super(agentId, agName, description);
		this.domainSizes = new HashMap<Integer, Integer>();
		this.counters = new HashMap<Integer, Long>();
	}
	/**
	 * 
	 */
	@Override
	public void addMsgHandlers() {
		super.addMsgHandlers();
		this.communicator.addMsgHandler(this.domSizeH);
	}
	/**
	 * 
	 */
	@Override
	public void checkAssign(int nextId) throws StopException {
		if (this.getAgentId() == nextId) {
			this.assignCPA();
		} else {
			this.sendDomMessages();
		}
	}
	/**
	 * 
	 */
	private void sendDomMessages() {
		IntArray unassigned = this.getOrder();
		unassigned = unassigned.minus(this.agentView.toInstantiation().getAgentIds());
		unassigned.remove(this.agentId); 
		for(int agId: unassigned){
			this.domSizeH.sendDomSizeMessage(agId);
		}
	}
	/**
	 * 
	 * @param cpa
	 * @return
	 */
	public int getNextAgent(Instantiation cpa) {
		IntArray unassigned = this.agentOrder.getLowerAgents();
		if (unassigned.size() == 0)
			return -1;
		switch(this.heuristic){
			case MIN_DOM: return getMinDom(unassigned);
			default : return getMinDomDeg(unassigned);
		}
	}
	
	/**
	 * 
	 * @param futureAgents
	 * @return
	 */
	public int getMinDom(IntArray futureAgents) {
		int minSize = Integer.MAX_VALUE;
		int nextAgent = -1;
		for(int agId : futureAgents){
			int dom = domainSizes.get(agId);
			if (minSize > dom) {
				minSize = dom;
				nextAgent = agId;
			}
		}
		return nextAgent;
	}
	
	/**
	 * 
	 * @param futureAgents
	 * @return
	 */
	public int getMinDomDeg(IntArray futureAgents) {
		double minDomDeg = Double.MAX_VALUE, domDeg;
		int nextAgent = -1;
		for(int agId : futureAgents){
			int dom = domainSizes.get(agId);
			int deg = this.getDeg(agId, futureAgents);
			if (deg == 0){
				domDeg = Double.POSITIVE_INFINITY;
			} else{
				domDeg = dom / deg;
			}
			if (minDomDeg > domDeg) {
				minDomDeg = domDeg;
				nextAgent = agId;
			}
		}
		return nextAgent;
	}
	
	/**
	 * 
	 * @param agId
	 * @param unassigned
	 * @return
	 */
	private int getDeg(int agId, IntArray unassigned) {
		if (heuristic == DOM_DEG) { return this.agentOrder.getAgentsGraph().degree(agId); }
		int future_deg = 0;
		for( int neigh : this.agentOrder.getNeighborsOf(agId)){
			if (unassigned.contains(neigh))
				future_deg++;
		}
		return future_deg;
	}

	// =============================================================================================
	// ////////////////////// DomMsgHandler //////////////////////////////////////////////////
	// =============================================================================================
	class DomMsgHandler extends MessageHandler {
		/**
		 * @param agent
		 * @param HandlerType
		 */
		public DomMsgHandler(AbstractAgent agent) {
			super(agent, Commands.DomainSize);
		}

		/* (non-Javadoc)
		 * @see kernel.com.MessageHandler#processMessage(kernel.com.Message)
		 */
		@Override
		public void processMessage(kernel.com.Message msg) throws kernel.StopException {
			int agId = msg.getSenderId();
			int dom = (Integer) msg.get(Commands.DomainSize + "");
			domainSizes.put(agId, dom);
		}
		
		public void sendDomSizeMessage(int receiverId) {
			Message msg = this.createMessage(); 
			msg.setReceiverId(receiverId);
			msg.addContents(Commands.DomainSize, var.getDomainSize()) ;
			sendMessage(msg);
		}
		
	}
//
//		static final String DOMAIN_SIZE = "domSize";
//		static final String COUNTER = "ctr";
//
//		public OrderMsgHandler(AFCngDomDegAgent agent) {
//			super(agent, Commands.ORDER);
//		}
//
//		public void processMessage(Message msg) throws StopException {
//			int sId = msg.getSenderId();
//			int domSize = (Integer) msg.get(DOMAIN_SIZE);
//			long counter = (Long) msg.get(COUNTER);
//			if (((AFCngDomDegAgent) this.myAgent).counters.containsKey(sId)) {
//				if (((AFCngDomDegAgent) this.myAgent).counters.get(sId) <= counter) {
//					((AFCngDomDegAgent) this.myAgent).counters.put(sId, counter);
//					((AFCngDomDegAgent) this.myAgent).domainSizes.put(sId, domSize);
//				}
//			} else {
//				((AFCngDomDegAgent) this.myAgent).counters.put(sId, counter);
//				((AFCngDomDegAgent) this.myAgent).domainSizes.put(sId, domSize);
//			}
//		}
//
//		// =============================================================================================
//
//		public void sendDomainSizeMsgs(Instantiation cpa) {
//			IntArray agIds = this.myAgent.agentOrder.getOrder();
//			IntArray unassigned = agIds.minus(cpa.getAgentIds());
//			unassigned.remove(this.myAgent.getAgentId());
//			for (int agId : unassigned) {
//				this.sendDomainSize(agId);
//			}
//		}
//
//		// =============================================================================================
//
//		public void sendDomainSize(int agId) {
//			((AFCngDomDegAgent) this.myAgent).myCounter++;
//			int domSize = ((AFCngDomDegAgent) this.myAgent).getMyVar().getDomainSize();
//			Message msg = createMessage();
//			msg.addContents(DOMAIN_SIZE, domSize);
//			msg.addContents(COUNTER, ((AFCngDomDegAgent) this.myAgent).myCounter);
//			msg.setReceiverId(agId);
//			this.myAgent.sendMessage(msg);
//		}
//	}

}